﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;

using Lucene.Net.Index;
using Lucene.Net.Store;
using LuceneDocs = Lucene.Net.Documents;  //namespace collision with DocumentFormat.OpenXml so I gave it an alias
using Lucene.Net.Analysis.Standard;
using Lucene.Net.Store.Azure;

using System.IO;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Wordprocessing;
using Lucene.Net.Search;
using Lucene.Net.QueryParsers;



namespace WorkerRole1
{
    public class WorkerRole : RoleEntryPoint
    {
        public override void Run()
        {
            // This is a sample worker implementation. Replace with your logic.
            Trace.WriteLine("WorkerRole1 entry point called", "Information");

            CloudStorageAccount.SetConfigurationSettingPublisher((configName,
                                                      configSettingPublisher) =>
            {
                string configValue = RoleEnvironment.GetConfigurationSettingValue(configName);
                configSettingPublisher(configValue);
            });

            Index(true);

            while (true)
            {
                Thread.Sleep(18000);
                Trace.WriteLine("Working", "Information");

                Index(false);
            }
        }

        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.

            return base.OnStart();
        }

        public void Index(bool DoFull)
        {
            CloudStorageAccount storageAccount = CloudStorageAccount.FromConfigurationSetting("StorageAccount");
            CloudBlobClient BlobClient = storageAccount.CreateCloudBlobClient();

            //open AzureDirectory which contains the index
            AzureDirectory azureDirectory = new AzureDirectory(storageAccount, "CloudIndex");

            ////need to check segments.gen last modified date and filter documents to index by that timestamp
            DateTime LastModified = new DateTime(IndexReader.LastModified(azureDirectory),DateTimeKind.Utc);
            bool GenerateIndex = !IndexReader.IndexExists(azureDirectory) ;
            DoFull = GenerateIndex;
                      
            //loop and fetch the information for each one
            //this needs to be managed for memory pressure, but for sample I'll do all in one pass
            IndexWriter indexWriter = new IndexWriter(azureDirectory, new StandardAnalyzer(Lucene.Net.Util.Version.LUCENE_29), GenerateIndex, IndexWriter.MaxFieldLength.UNLIMITED);

            BlobRequestOptions options = new BlobRequestOptions();
            options.BlobListingDetails = BlobListingDetails.Metadata;
            options.UseFlatBlobListing = true;

            //get container to be indexed
            CloudBlobContainer Container = BlobClient.GetContainerReference("documents");
            Container.CreateIfNotExist();

            foreach (IListBlobItem currentBlob in Container.ListBlobs(options))
            {
                CloudBlob blobRef = Container.GetBlobReference(currentBlob.Uri.ToString());
                blobRef.FetchAttributes(options);
                //add doc to index if it is newer than index or doing a full index
                if (LastModified < blobRef.Properties.LastModifiedUtc || DoFull )
                {
                    DocumentToIndex curBlob = GetDocumentData(currentBlob.Uri.ToString());
                    
                    //docs.Add(curBlob);

                    AddToCatalog(indexWriter, curBlob);
                }
            }

            if (DoFull)
            {
                indexWriter.Optimize();
            }

            indexWriter.Commit();
            indexWriter.Close();
 

        }

        public DocumentToIndex GetDocumentData(string docUri)
        {
            DocumentToIndex doc = new DocumentToIndex();

            //Stream stream = File.Open(docUri, FileMode.Open);
            var response = WebRequest.Create(docUri).GetResponse();
            Stream stream = response.GetResponseStream();

            //can't open directly from URI, because it won't support seeking
            //so move it to a "local" memory stream
            Stream localStream= new MemoryStream();
            stream.CopyTo(localStream);

            //parse doc name
            doc.Name = docUri.Substring(docUri.LastIndexOf(@"/")+1);
            doc.Uri = docUri;

            switch(doc.Name.Substring(doc.Name.LastIndexOf(".")+1))
            {
                case "docx":
                    WordprocessingDocument wordprocessingDocument = WordprocessingDocument.Open(localStream, false);
                    doc.Body = wordprocessingDocument.MainDocumentPart.Document.Body.InnerText;
                    wordprocessingDocument.Close();
                    break;
                //TODO:  Still incomplete
                case "pptx":
                    //probably want to create a generic fo DocToIndex and use it to create a pptx specific that allows slide 
                    //specific indexing
                    PresentationDocument pptDoc = PresentationDocument.Open(localStream, false);
                    foreach (SlidePart slide in pptDoc.PresentationPart.SlideParts)
                    {
                        //iterate through slides
                    }
                    break;
                default:
                    break;

            }


            stream.Close();
            return doc;
        }

        public void RemoveFromCatalog(List<DocumentToIndex> docs, AzureDirectory azureDir)
        {

            IndexReader indexReader = IndexReader.Open(azureDir);
            foreach (DocumentToIndex currentDocument in docs)
            {

                Term deleteTerm = new Term("Uri", currentDocument.Uri);
                long deletedCount = indexReader.DeleteDocuments(deleteTerm);

            }
      
            indexReader.Commit();
            indexReader.Close();
        }

        public void AddToListCatalog(IndexWriter indexWriter, List<DocumentToIndex> docs)
        {
            foreach (DocumentToIndex currentDocument in docs)
            {
                AddToCatalog(indexWriter, currentDocument);
            }

        }

        public void AddToCatalog(IndexWriter indexWriter, DocumentToIndex currentDocument)
        {

            Term deleteTerm = new Term("Uri", currentDocument.Uri);

            LuceneDocs.Document doc = new LuceneDocs.Document();
            doc.Add(new LuceneDocs.Field("Uri", currentDocument.Uri, LuceneDocs.Field.Store.YES, LuceneDocs.Field.Index.NOT_ANALYZED, LuceneDocs.Field.TermVector.NO));
            doc.Add(new LuceneDocs.Field("Title", currentDocument.Name, LuceneDocs.Field.Store.YES, LuceneDocs.Field.Index.ANALYZED, LuceneDocs.Field.TermVector.NO));
            doc.Add(new LuceneDocs.Field("Body", currentDocument.Body, LuceneDocs.Field.Store.YES, LuceneDocs.Field.Index.ANALYZED, LuceneDocs.Field.TermVector.NO));

            indexWriter.UpdateDocument(deleteTerm, doc);
        }

       
    }

    public class DocumentToIndex
    {
        public string Body;
        public string Name;
        public string Uri;
        public string Id;
    }
    
}
